/*
 * SmartFoxServer 2X + RedBox Examples - VideoChat
 * http://www.smartfoxserver.com
 * (c) 2011 gotoAndPlay
 */
import com.smartfoxserver.smartfoxbits.events.BitEvent;
import com.smartfoxserver.v2.SmartFox;
import com.smartfoxserver.v2.core.SFSEvent;
import com.smartfoxserver.v2.entities.User;
import com.smartfoxserver.v2.entities.data.ISFSObject;
import com.smartfoxserver.v2.entities.data.SFSObject;
import com.smartfoxserver.v2.redbox.AVChatManager;
import com.smartfoxserver.v2.redbox.data.ChatSession;
import com.smartfoxserver.v2.redbox.events.RedBoxChatEvent;
import com.smartfoxserver.v2.redbox.events.RedBoxConnectionEvent;
import com.smartfoxserver.v2.requests.CreateRoomRequest;
import com.smartfoxserver.v2.requests.PrivateMessageRequest;
import com.smartfoxserver.v2.requests.RoomSettings;
import com.smartfoxserver.v2.util.ClientDisconnectionReason;

import components.CreateRoomPanel;
import components.PrivateChatTab;

import flexlib.events.SuperTabEvent;

import mx.controls.Alert;
import mx.controls.Button;
import mx.events.CloseEvent;
import mx.managers.PopUpManager;


private var sfs:SmartFox;
private var alert:Alert;
private var createRoomPanel:CreateRoomPanel;
private var avChatMan:AVChatManager;

private function init():void
{
	// Get reference to SmartFoxServer connection
	sfs = loginPanel.connector.connection;
	sfs.addEventListener(SFSEvent.LOGIN, onLogin);
	sfs.addEventListener(SFSEvent.CONNECTION_LOST, onConnectionLost);
	sfs.addEventListener(SFSEvent.ROOM_CREATION_ERROR, onRoomCreationError);
	sfs.addEventListener(SFSEvent.PRIVATE_MESSAGE, onPrivateMessage);
	
	ul_users.addEventListener(BitEvent.USER_CHANGE, onUserChange);
}

//---------------------------------
// User interaction event handlers
//---------------------------------

/**
 * Create room button click event listener.
 * Show a popup panel where user can set the room properties.
 */
private function onCreateRoomApplicationBtClick():void
{
	createRoomPanel = PopUpManager.createPopUp(this, CreateRoomPanel, true) as CreateRoomPanel;
	PopUpManager.centerPopUp(createRoomPanel);
	createRoomPanel.addEventListener(CloseEvent.CLOSE, onPopUpClosed, false, 0, true);
}

/**
 * Create room button click event listener.
 * Create a new room using the parameters entered in the create room pupup. 
 */
public function onCreateRoomPopuUpBtClick():void
{
	if (createRoomPanel.ti_roomName.length > 0)
	{
		// Collect room settings
		var settings:RoomSettings = new RoomSettings(createRoomPanel.ti_roomName.text);
		settings.groupId = createRoomPanel.lb_group.text;
		settings.maxUsers = createRoomPanel.ns_maxUsers.value;
		
		// Create room
		var request:CreateRoomRequest = new CreateRoomRequest(settings, true, sfs.lastJoinedRoom);
		sfs.send(request);
		
		// Close popup
		removePopUp();
	}
}

private function onAlertClosed(evt:Event):void
{
	removeAlert();
}

private function onPopUpClosed(evt:Event):void
{
	removePopUp();
}

private function onUserChange(evt:BitEvent):void
{
	bt_startPrivateChat.enabled = (ul_users.list.selectedIndex > -1);
}

/**
 * Start a new private chat with the selected user.
 */
private function onStartPrivChatBtClick():void
{
	if (ul_users.list.selectedIndex > -1)
	{
		var user:User = ul_users.list.selectedItem.userData;
		var tab:PrivateChatTab = getPrivateChatTab(user.id, user.name);
		stn_chats.selectedChild = tab;
	}
}

/**
 * Close private chat panel. 
 */
public function onPrivChatCloseBtClick(evt:SuperTabEvent):void
{
	var tab:PrivateChatTab = stn_chats.getChildAt(evt.tabIndex) as PrivateChatTab;
	stopAVConnInChatTab(tab);
}

/**
 * Handle click on the "Send" button in the private chat panel.
 */
public function onPrivateMsgSendBtClick(evt:Event):void
{
	var panel:PrivateChatTab = (evt.target as Button).parentDocument as PrivateChatTab
	
	if (panel.ti_message.text != "")
	{
		var params:ISFSObject = new SFSObject();
		params.putInt("recipientId", panel.userId);
		
		var request:PrivateMessageRequest = new PrivateMessageRequest(panel.ti_message.text, panel.userId, params);
		sfs.send(request); 
		
		panel.ti_message.text = "";
	}
}

/**
 * Handle "start A/V chat" button click in the private chat panel.
 */
public function onStartAVChatBtClick(panel:PrivateChatTab):void
{
	if (avChatMan != null && avChatMan.isConnected)
	{
		var session:ChatSession = avChatMan.sendChatRequest(AVChatManager.REQ_TYPE_SEND_RECEIVE, panel.userId, true, true);
		
		if (session != null) // session is null if the same request to the same recipient has already been submitted before (and is still valid)
		{
			panel.connId = session.id;
			panel.showMyVideo(Camera.getCamera());
			enableStartAVButton(panel, false);
			showSystemMessage(panel, "You have invited " + panel.userName + " to start an A/V chat");
		}
	}
	else
		showSystemMessage(panel, "Unable to send an A/V chat request: the connection to RED5 server could not be established on application startup");
}

/**
 * Handle "stop A/V chat" button click in the private chat panel.
 */
public function onStopAVChatBtClick(panel:PrivateChatTab):void
{
	// Send stop request
	avChatMan.stopChat(panel.connId);
	
	// Re-enable proper a/v buttons, remove my video if necessary and display system message in chat box
	panel.connId = "";
	panel.stopMyVideo();
	panel.stopFriendVideo();
	enableStartAVButton(panel, true);
	showSystemMessage(panel, "You have stopped the current A/V chat");
}

/**
 * Handle "accept A/V chat" button click in the private chat panel.
 */
public function onAcceptAVRequestBtClick(panel:PrivateChatTab):void
{
	if (avChatMan != null && avChatMan.isConnected)
	{
		try
		{
			// Send acceptance
			avChatMan.acceptChatRequest(panel.connId);
			
			// Reset the private chat panel current status (to hide invitation)
			panel.currentState = "";
			
			// Get the A/V session data
			var session:ChatSession = avChatMan.getChatSession(panel.connId);
			
			if (session.enableCamera)
				panel.showMyVideo(Camera.getCamera());
			
			// Disable a/v buttons and display system message in chat box
			enableStartAVButton(panel, false);
			showSystemMessage(panel, "You have accepted to start the A/V chat");
		}
		catch (e:Error)
		{
			showAlert("The following error occurred while accepting the A/V chat request: " + e.message);
		}
	}
	else
		showSystemMessage(panel, "Unable to accept an A/V chat request: the connection to RED5 server could not be established on application startup");
}

/**
 * Handle "decline A/V chat" button click in the private chat panel.
 */
public function onDeclineAVRequestBtClick(panel:PrivateChatTab):void
{
	try
	{
		// Send refusal
		avChatMan.refuseChatRequest(panel.connId);
		
		// Reset the private chat panel current status (to hide invitation)
		panel.currentState = "";
		
		// Re-enable a/v buttons and display system message in chat box
		enableStartAVButton(panel, true);
		showSystemMessage(panel, "You have refused to start the A/V chat");
		panel.connId = "";
	}
	catch (e:Error)
	{
		showAlert("The following error occurred while declining the A/V chat request: " + e.message);
	}
}

//---------------------------------
// SmartFoxServer event handlers
//---------------------------------

/**
 * On login, show the chat view.
 */
private function onLogin(evt:SFSEvent):void
{
	// Move to chat view, and display user name 
	mainView.selectedChild = view_chat;
	lb_myUserName.text = sfs.mySelf.name;
	
	// Initialize AVChatManager
	initializeAV();
}

/**
 * On connection lost, go back to login panel view and display disconnection error message.
 */
private function onConnectionLost(evt:SFSEvent):void
{
	// Remove popup, if any
	removePopUp();
	
	// Remove alert, if displayed
	removeAlert();
	
	// Close chat tabs
	for each (var tab:PrivateChatTab in stn_chats.getChildren())
	{
		stopAVConnInChatTab(tab);
		stn_chats.removeChild(tab);
	}
	
	// Remove listeners added to AVChatManager instance
	if (avChatMan != null)
	{
		avChatMan.removeEventListener(RedBoxConnectionEvent.AV_CONNECTION_INITIALIZED, onAVConnectionInited);
		avChatMan.removeEventListener(RedBoxConnectionEvent.AV_CONNECTION_ERROR, onAVConnectionError);
		avChatMan.removeEventListener(RedBoxChatEvent.CHAT_REQUEST, onChatRequest);
		avChatMan.removeEventListener(RedBoxChatEvent.CHAT_REFUSED, onChatRefused);
		avChatMan.removeEventListener(RedBoxChatEvent.CHAT_STARTED, onChatStarted);
		avChatMan.removeEventListener(RedBoxChatEvent.CHAT_STOPPED, onChatStopped);
		avChatMan = null;
	}
	
	// Show disconnection message, unless user chose voluntarily to close the connection
	if (evt.params.reason != ClientDisconnectionReason.MANUAL)
	{
		var msg:String = "Connection lost";
		
		switch (evt.params.reason)
		{
			case ClientDisconnectionReason.IDLE:
				msg += "\nYou have exceeded the maximum user idle time";
				break;
			
			case ClientDisconnectionReason.KICK:
				msg += "\nYou have been kicked";
				break;
			
			case ClientDisconnectionReason.BAN:
				msg += "\nYou have been banned";
				break;
			
			case ClientDisconnectionReason.UNKNOWN:
				msg += " due to unknown reason\nPlease check the server-side log";
				break;
		}
		
		loginPanel.ta_error.text = msg;
	}
	
	// Show login view
	mainView.selectedChild = view_connecting;
}

/**
 * On room creation error, show an alert.
 */
private function onRoomCreationError(evt:SFSEvent):void
{
	// Show alert
	showAlert("Unable to create room due to the following error: " + evt.params.errorMessage);
}

/**
 * Handle private message.
 */
private function onPrivateMessage(evt:SFSEvent):void
{
	var sender:User = evt.params.sender;
	var params:ISFSObject = evt.params.data;
	
	if (params.containsKey("recipientId"))
	{
		var recipientId:int = params.getInt("recipientId");
		var message:String = evt.params.message;
		
		// Retrieve the proper private chat tab: if I'm the sender of the message, then I can use
		// the recipientId value contained in the message body; otherwise I can use my own id
		var tab:PrivateChatTab = (sender == sfs.mySelf ? getPrivateChatTab(recipientId, null) : getPrivateChatTab(sender.id, sender.name));
		showPrivateMessage(tab, sender.name, message);
	}
}

//---------------------------------------------------------------------
// RedBox AVChatManager event handlers
//---------------------------------------------------------------------

/**
 * Handle A/V connection initialized.
 */
public function onAVConnectionInited(evt:RedBoxConnectionEvent):void
{
	// Nothing to do. Usually we should wait this event before enabling the a/v chat related interface elements.
}

/**
 * Handle A/V connection error.
 */
public function onAVConnectionError(evt:RedBoxConnectionEvent):void
{
	var error:String = "The following error occurred while trying to establish an A/V connection: " + evt.params.errorCode;
	
	// Show alert
	showAlert(error);
	
	// Re-enable proper a/v buttons, remove video if necessary and display system message in chat box
	for each (var tab:PrivateChatTab in stn_chats.getChildren())
	{
		enableStartAVButton(tab, true);
		tab.stopMyVideo();
		tab.stopFriendVideo();
		
		showSystemMessage(tab, error);
	}
}

/**
 * Handle incoming A/V chat request.
 */
public function onChatRequest(evt:RedBoxChatEvent):void
{
	var chatSession:ChatSession = evt.params.chatSession;
	
	trace("A/V CHAT REQUEST RECEIVED:");
	trace(chatSession.toString());
	
	var senderId:int = chatSession.mateId;
	var senderName:String = chatSession.mateName;
	
	// Retrieve the chat tab
	var tab:PrivateChatTab = getPrivateChatTab(senderId, senderName);
	
	// Set the A/V connection id for future reference; show the invitation message and accept/refuse buttons
	tab.connId = chatSession.id;
	tab.currentState = "invitation";
	showSystemMessage(tab, senderName + " invited you to start an A/V chat");
}

/**
 * Handle A/V chat started event.
 */
public function onChatStarted(evt:RedBoxChatEvent):void
{
	trace("AV Chat started")
	
	var chatSession:ChatSession = evt.params.chatSession;
	
	var userId:int = chatSession.mateId;
	var userName:String = chatSession.mateName;
	
	// Retrieve the chat tab
	var tab:PrivateChatTab = getPrivateChatTab(userId, userName);
	
	// Attach the NetStream object to the video object
	if (chatSession.mateStream != null)
		tab.showFriendVideo(chatSession.mateStream);
	
	// Write a system message in the chat box (only if I'm the requester)
	if (chatSession.iAmRequester)
		showSystemMessage(tab, userName + " accepted your invitation");
}

/**
 * Handle A/V chat refused event.
 */
public function onChatRefused(evt:RedBoxChatEvent):void
{
	trace("AV Connection refused")
	
	var chatSession:ChatSession = evt.params.chatSession;
	var userId:int = chatSession.mateId;
	var userName:String = chatSession.mateName;
	
	// Retrieve the chat panel
	var tab:PrivateChatTab = getPrivateChatTab(userId, userName);
	
	// Write a system message in the chat box, re-enable buttons and stop my own video
	tab.stopMyVideo();
	enableStartAVButton(tab, true);
	tab.connId = "";
	showSystemMessage(tab, "Your invitation was declined");
}

/**
 * Handle A/V chat stopped event.
 */
public function onChatStopped(evt:RedBoxChatEvent):void
{
	trace("AV Connection stopped");
	
	var chatSession:ChatSession = evt.params.chatSession;
	var userId:int = chatSession.mateId;
	var userName:String = chatSession.mateName;
	
	// Retrieve the chat panel
	var tab:PrivateChatTab = getPrivateChatTab(userId, userName);
	
	// Write a system message in the chat box, re-enable buttons and stop my own video
	tab.currentState = "";
	tab.stopMyVideo();
	tab.stopFriendVideo();
	enableStartAVButton(tab, true);
	tab.connId = "";
	showSystemMessage(tab, userName + " stopped the A/V chat");
}

//---------------------------------
// Other methods
//---------------------------------

/**
 * Initialize the AVChatManager instance.
 */
private function initializeAV():void
{
	// Create AVChatmanager instance
	avChatMan = new AVChatManager(sfs, sfs.currentIp, false, true);
	
	avChatMan.addEventListener(RedBoxConnectionEvent.AV_CONNECTION_INITIALIZED, onAVConnectionInited);
	avChatMan.addEventListener(RedBoxConnectionEvent.AV_CONNECTION_ERROR, onAVConnectionError);
	avChatMan.addEventListener(RedBoxChatEvent.CHAT_REQUEST, onChatRequest);
	avChatMan.addEventListener(RedBoxChatEvent.CHAT_REFUSED, onChatRefused);
	avChatMan.addEventListener(RedBoxChatEvent.CHAT_STARTED, onChatStarted);
	avChatMan.addEventListener(RedBoxChatEvent.CHAT_STOPPED, onChatStopped);
}

/**
 * Retrieve the private chat tab.
 * If not existing, create it.
 */
private function getPrivateChatTab(userId:int, userName:String):PrivateChatTab
{
	var tab:PrivateChatTab = stn_chats.getChildByName("user_" + userId) as PrivateChatTab;
	
	if (tab == null)
	{
		tab = new PrivateChatTab();
		tab.name = "user_" + userId;
		
		tab.userId = userId;
		tab.userName = userName;
		tab.label = userName;
		
		stn_chats.addChild(tab);
	}
	
	return tab;
}

/**
 * Remove the private chat tab.
 */
private function stopAVConnInChatTab(tab:PrivateChatTab):void
{
	// Stop A/V connection
	if (tab.connId != null)
		onStopAVChatBtClick(tab);
}

/**
 * Show user message in private chat tab.
 */
private function showPrivateMessage(tab:PrivateChatTab, senderName:String, txt:String):void
{
	tab.ta_chat.htmlText += "<b><font color='#F88121'>" + senderName + ": </font></b>" + txt;
	tab.ta_chat.verticalScrollPosition = tab.ta_chat.maxVerticalScrollPosition;
}

/**
 * Show system message in private chat tab.
 */
private function showSystemMessage(tab:PrivateChatTab, txt:String):void
{
	tab.ta_chat.htmlText += "<i><font color='#666666'>" + txt + "</font></i>\n";
	tab.ta_chat.verticalScrollPosition = tab.ta_chat.maxVerticalScrollPosition;
}

/**
 * Enable/disable A/V buttons in a private chat tab.
 */
private function enableStartAVButton(tab:PrivateChatTab, enable:Boolean):void
{
	tab.bt_startAV.enabled = enable;
	tab.bt_stopAV.enabled = !enable;
}

private function showAlert(message:String):void
{
	// Remove previous alert, if any
	removeAlert();
	
	// Show alert
	alert = Alert.show(message, "Warning", Alert.OK, null, onAlertClosed);
}

private function removeAlert():void
{
	if (alert != null)
		PopUpManager.removePopUp(alert);
	
	alert = null;
}

private function removePopUp():void
{
	if (createRoomPanel != null)
	{
		createRoomPanel.removeEventListener(CloseEvent.CLOSE, onPopUpClosed);
		PopUpManager.removePopUp(createRoomPanel);
		createRoomPanel = null;
	}
}