/*
 * SmartFoxServer 2X + RedBox Examples - VideoClipPlayer
 * http://www.smartfoxserver.com
 * (c) 2011 gotoAndPlay
 */
import com.smartfoxserver.v2.SmartFox;
import com.smartfoxserver.v2.core.SFSEvent;
import com.smartfoxserver.v2.redbox.AVClipManager;
import com.smartfoxserver.v2.redbox.data.Clip;
import com.smartfoxserver.v2.redbox.events.RedBoxClipEvent;
import com.smartfoxserver.v2.redbox.events.RedBoxConnectionEvent;
import com.smartfoxserver.v2.redbox.exceptions.ClipActionNotAllowedException;
import com.smartfoxserver.v2.redbox.exceptions.NoAVConnectionException;
import com.smartfoxserver.v2.requests.JoinRoomRequest;
import com.smartfoxserver.v2.util.ClientDisconnectionReason;

import flash.events.NetStatusEvent;
import flash.media.Camera;

import mx.collections.ArrayCollection;
import mx.collections.IViewCursor;
import mx.controls.Alert;
import mx.managers.PopUpManager;

private var sfs:SmartFox;
private var avClipMan:AVClipManager;
private var alert:Alert;

[Bindable]
private var clipsDP:ArrayCollection;

private function init():void
{
	// Get reference to SmartFoxServer connection
	sfs = loginPanel.connector.connection;
	sfs.addEventListener(SFSEvent.LOGIN, onLogin);
	sfs.addEventListener(SFSEvent.CONNECTION_LOST, onConnectionLost);
}

//---------------------------------
// User interaction event handlers
//---------------------------------

/**
 * Called by the datagrid component when the selected item is changed
 */
public function onClipSelected():void
{
	// Get the selected clip id
	var clipId:String = dg_clips.selectedItem.id;
	
	// Request a stream object to AVClipManager instance
	try
	{
		var stream:NetStream = avClipMan.getStream();
		
		// Add event listeners to stream
		stream.addEventListener(NetStatusEvent.NET_STATUS, onNetStatus);
		stream.client = this; // this is required in order to handle the onMetaData, onCuePoint and onPlayStatus events
		
		// Play stream
		videoPlayer.reset();
		videoPlayer.playStream(stream, clipId);
		
		tb_title.text = dg_clips.selectedItem.title;
		tb_author.text = dg_clips.selectedItem.author;
		
		// Enable buttons
		bt_delete.enabled = true;
		bt_update.enabled = true;
	}
	catch (err:NoAVConnectionException)
	{
		showAlert("The following error occurred when trying to play the selected videoclip: " + err.message);
	}
}

/**
 * Delete the selected clip
 */
public function onDeleteClipBtClick():void
{
	// Get the selected clip id
	var clipId:String = dg_clips.selectedItem.id;
	
	try
	{
		avClipMan.deleteClip(clipId);
	}
	catch (err:ClipActionNotAllowedException)
	{
		showAlert("The following error occurred when trying to delete the current videoclip: " + err.message);
	}
}

/**
 * Update the selected clip's properties
 */
public function onUpdateClipBtClick():void
{
	// Get the selected clip id
	var clipId:String = dg_clips.selectedItem.id;
	
	try
	{
		var clipProperties:Object = {};
		clipProperties.title = tb_title.text;
		clipProperties.author = tb_author.text;
		
		avClipMan.updateClipProperties(clipId, clipProperties);
	}
	catch (err:ClipActionNotAllowedException)
	{
		showAlert("The following error occurred when trying to update the current videoclip: " + err.message);
	}
}

/**
 * Called when the accordion view changes
 */
public function onAccordionViewChange():void
{
	resetPlayBackInterface();
	resetRecordingInterface();
}

/**
 * Start recording a new clip
 */
public function onRecordBtClick():void
{
	avClipMan.startClipRecording(true, true);
}

/**
 * Stop clip recording
 */
public function onStopBtClick():void
{
	avClipMan.stopClipRecording();
	
	// Clear video container
	sv_video.video.attachCamera(null);
}

/**
 * Cancel clip recording
 */
public function onCancelBtClick():void
{
	// Reset a/v clip recording interface (recording is cancelled too)
	resetRecordingInterface();
	
	// Go back to playback section
	accordion.selectedIndex = 0;
}

/**
 * Preview recorded clip
 */
public function onPreviewBtClick():void
{
	var stream:NetStream = avClipMan.previewRecordedClip();
	sv_video.video.attachNetStream(stream);
}

/**
 * Submit recorded clip
 */
public function onSubmitBtClick():void
{
	var clipProperties:Object = {};
	clipProperties.title = tb_titleRec.text;
	clipProperties.author = tb_authorRec.text;
	
	avClipMan.submitRecordedClip(clipProperties);
	
	onCancelBtClick();
}

private function onAlertClosed(evt:Event):void
{
	removeAlert();
}

//---------------------------------
// SmartFoxServer event handlers
//---------------------------------

/**
 * On login, show the main view.
 */
private function onLogin(evt:SFSEvent):void
{
	// Move to main view, and display user name 
	mainView.selectedChild = view_chat;
	lb_myUserName.text = sfs.mySelf.name;
	
	// Join the lobby, to allow public chat
	var request:JoinRoomRequest = new JoinRoomRequest("The Lobby");
	sfs.send(request);
	
	// Initialize AVClipManager
	initializeAV();
}

/**
 * On connection lost, go back to login panel view and display disconnection error message.
 */
private function onConnectionLost(evt:SFSEvent):void
{
	// Remove alert, if displayed
	removeAlert();
	
	// Reset a/v clip playback interface
	resetPlayBackInterface();
	
	// Reset a/v clip recording interface
	resetRecordingInterface();
	
	// Remove listeners added to AVClipManager instance
	if (avClipMan != null)
	{
		avClipMan.removeEventListener(RedBoxConnectionEvent.AV_CONNECTION_INITIALIZED, onAVConnectionInited);
		avClipMan.removeEventListener(RedBoxConnectionEvent.AV_CONNECTION_ERROR, onAVConnectionError);
		avClipMan.removeEventListener(RedBoxClipEvent.CLIP_LIST, onAVClipList);
		avClipMan.removeEventListener(RedBoxClipEvent.CLIP_SUBMISSION_FAILED, onClipSubmissionFailed);
		avClipMan.removeEventListener(RedBoxClipEvent.CLIP_ADDED, onClipAdded);
		avClipMan.removeEventListener(RedBoxClipEvent.CLIP_RECORDING_STARTED, onClipRecordingStarted);
		avClipMan.removeEventListener(RedBoxClipEvent.CLIP_DELETED, onClipDeleted);
		avClipMan.removeEventListener(RedBoxClipEvent.CLIP_UPDATED, onClipUpdated);
		avClipMan = null;
	}
	
	// Show disconnection message, unless user chose voluntarily to close the connection
	if (evt.params.reason != ClientDisconnectionReason.MANUAL)
	{
		var msg:String = "Connection lost";
		
		switch (evt.params.reason)
		{
			case ClientDisconnectionReason.IDLE:
				msg += "\nYou have exceeded the maximum user idle time";
				break;
			
			case ClientDisconnectionReason.KICK:
				msg += "\nYou have been kicked";
				break;
			
			case ClientDisconnectionReason.BAN:
				msg += "\nYou have been banned";
				break;
			
			case ClientDisconnectionReason.UNKNOWN:
				msg += " due to unknown reason\nPlease check the server-side log";
				break;
		}
		
		loginPanel.ta_error.text = msg;
	}
	
	// Show login view
	mainView.selectedChild = view_connecting;
}

//---------------------------------------------------------------------
// RedBox AVClipManager event handlers
//---------------------------------------------------------------------

/**
 * Handle A/V connection initialized.
 */
public function onAVConnectionInited(evt:RedBoxConnectionEvent):void
{
	// Nothing to do. Usually we should wait this event before enabling the a/v chat related interface elements.
}

/**
 * Handle A/V connection error.
 */
public function onAVConnectionError(evt:RedBoxConnectionEvent):void
{
	var error:String = "The following error occurred while trying to establish an A/V connection: " + evt.params.errorCode;
	
	// Show alert
	showAlert(error);
}

public function onAVClipList(evt:RedBoxClipEvent):void
{
	clipsDP.removeAll();
	
	for each (var clip:Clip in evt.params.clipList)
		addClipToList(clip);
}

public function onClipSubmissionFailed(evt:RedBoxClipEvent):void
{
	// Show alert
	showAlert("The following error occurred when trying to submit a videoclip: " + evt.params.error);
}

public function onClipAdded(evt:RedBoxClipEvent):void
{
	// Update clips list
	addClipToList(evt.params.clip);
}

public function onClipRecordingStarted(evt:RedBoxClipEvent):void
{
	// Attach camera stream to video container on the stage
	sv_video.video.attachCamera(Camera.getCamera());
}

public function onClipDeleted(evt:RedBoxClipEvent):void
{
	// Update clips list
	var i:int = searchClipsList(evt.params.clip.id);
	
	if (dg_clips.selectedItem.id == evt.params.clip.id)
		resetPlayBackInterface();
	
	if (i > -1)
		clipsDP.removeItemAt(i);
}

public function onClipUpdated(evt:RedBoxClipEvent):void
{
	// Update clips list
	var clip:Clip = evt.params.clip;
	var i:int = searchClipsList(clip.id);
	
	if (i > -1)
	{
		var item:Object = clipsDP.getItemAt(i);
		item.title = clip.properties.title;
		item.author = clip.properties.author;
		
		clipsDP.refresh();
		
		if (dg_clips.selectedItem.id == clip.id)
		{
			tb_author.text = clip.properties.author;
			tb_title.text = clip.properties.title;
		}
	}
}

//---------------------------------------------------------------------
// NetStream event handlers
//---------------------------------------------------------------------

public function onNetStatus(evt:NetStatusEvent):void
{
	trace ("--- onNetStatus event");
	trace ("\tLevel: " + evt.info.level);
	trace ("\tCode: " + evt.info.code);
}

public function onMetaData(info:Object):void
{
	trace ("--- onMetaData Event");
	trace ("\tDuration: " + info.duration);
	trace ("\tFramerate: " + info.framerate);
	trace ("\tWidth: " + info.width);
	trace ("\tHeight: " + info.height);
	trace ("\tCue points: " + info.cuePoints);
	
	// Set clip duration (to enable seek bar)
	if (info.duration > 0)
		videoPlayer.setDuration(info.duration);
	
	// Set clip aspect ratio
	if (info.width != undefined && info.height != undefined)
		videoPlayer.setAspectRatio(info.width / info.height);
	
	// Display metadata
	lb_duration.text = (info.duration > 0 ? (Math.round(info.duration) + " seconds") : "unknown");
	lb_framerate.text = (info.framerate != undefined ? (Math.round(info.framerate) + " fps") : "unknown");
	lb_size.text = ((info.width != undefined && info.height != undefined) ? (info.width + "x" + info.height + " pixels") : "unknown");
}

public function onCuePoint(info:Object):void
{
	trace ("--- onCuePoint event");
	trace ("\tName: " + info.name);
	trace ("\tParameters: " + info.parameters);
	trace ("\tTime: " + info.time);
	trace ("\tType: " + info.type);
}

public function onPlayStatus(info:Object):void
{
	trace ("--- onPlayStatus event");
	trace ("\tLevel: " + info.level);
	trace ("\tCode: " + info.code);
	
	// Reset video upon clip completion
	if (info.code == "NetStream.Play.Complete")
		videoPlayer.stopStream();
}

public function onLastSecond(info:Object):void
{
	trace ("--- onLastSecond event");
	
	// Some streams also fire this event
}

//---------------------------------
// Other methods
//---------------------------------

/**
 * Initialize the AVChatManager instance.
 */
private function initializeAV():void
{
	// Init clips datagrid's dataprovider
	clipsDP = new ArrayCollection();
	
	// Create AVChatmanager instance
	avClipMan = new AVClipManager(sfs, sfs.currentIp, false, true);
	
	avClipMan.addEventListener(RedBoxConnectionEvent.AV_CONNECTION_INITIALIZED, onAVConnectionInited);
	avClipMan.addEventListener(RedBoxConnectionEvent.AV_CONNECTION_ERROR, onAVConnectionError);
	avClipMan.addEventListener(RedBoxClipEvent.CLIP_LIST, onAVClipList);
	avClipMan.addEventListener(RedBoxClipEvent.CLIP_SUBMISSION_FAILED, onClipSubmissionFailed);
	avClipMan.addEventListener(RedBoxClipEvent.CLIP_ADDED, onClipAdded);
	avClipMan.addEventListener(RedBoxClipEvent.CLIP_RECORDING_STARTED, onClipRecordingStarted);
	avClipMan.addEventListener(RedBoxClipEvent.CLIP_DELETED, onClipDeleted);
	avClipMan.addEventListener(RedBoxClipEvent.CLIP_UPDATED, onClipUpdated);
	
	// Retrieve clips list
	avClipMan.getClipList();
}

private function addClipToList(clip:Clip):void
{
	var tmpObj:Object = {};
		
	tmpObj.id = clip.id;
	tmpObj.rtmp = clip.rtmpURL;
	tmpObj.submitter = clip.username;
	tmpObj.size = Math.round(clip.size / 1000) + " Kb";
	tmpObj.published = clip.lastModified;
	tmpObj.title = clip.properties.title;
	tmpObj.author = clip.properties.author;
	
	clipsDP.addItem(tmpObj);
}

private function resetPlayBackInterface():void
{
	// Reset video player
	videoPlayer.reset();
	
	// Reset datagrid
	dg_clips.selectedIndex = -1;
	
	// Disable delete/update buttons
	bt_delete.enabled = false;
	bt_update.enabled = false;
	
	// Reset textfields
	lb_duration.text = "---";
	lb_framerate.text = "---";
	lb_size.text = "---";
	tb_title.text = "";
	tb_author.text = "";
}

private function resetRecordingInterface():void
{
	// Cancel recording (in case it's in progress)
	avClipMan.cancelClipRecording();
	
	// Reset textfields
	tb_titleRec.text = "";
	tb_authorRec.text = "";
	
	// Clear video container in recording section
	sv_video.video.attachNetStream(null);
	sv_video.video.attachCamera(null);
	sv_video.video.clear(); // <-- NOT WORKING (possible Flash Player bug: only 1 pixel in the top left corner is cleared)
	
	sv_video.reset(); // As the previous statement doesn't work, we reset the video container
}

/**
 * Search the clips dataprovider and return the item index.
 */
private function searchClipsList(clipId:String):int
{
	// Declare a cursor
	var cursor:IViewCursor = clipsDP.createCursor();
	
	// Use cursor to search for clip id
	while (!cursor.afterLast)
	{
		if (cursor.current.id == clipId)
			return cursor.bookmark.getViewIndex();
		
		cursor.moveNext();
	}
	
	return -1;
}

private function showAlert(message:String):void
{
	// Remove previous alert, if any
	removeAlert();
	
	// Show alert
	alert = Alert.show(message, "Warning", Alert.OK, null, onAlertClosed);
}

private function removeAlert():void
{
	if (alert != null)
		PopUpManager.removePopUp(alert);
	
	alert = null;
}