/*
 * SmartFoxServer 2X + RedBox Examples - VideoMessenger
 * http://www.smartfoxserver.com
 * (c) 2011 gotoAndPlay
 */
import com.smartfoxserver.v2.SmartFox;
import com.smartfoxserver.v2.core.SFSBuddyEvent;
import com.smartfoxserver.v2.core.SFSEvent;
import com.smartfoxserver.v2.entities.Buddy;
import com.smartfoxserver.v2.entities.SFSBuddy;
import com.smartfoxserver.v2.entities.data.ISFSObject;
import com.smartfoxserver.v2.entities.data.SFSObject;
import com.smartfoxserver.v2.entities.variables.BuddyVariable;
import com.smartfoxserver.v2.entities.variables.ReservedBuddyVariables;
import com.smartfoxserver.v2.entities.variables.SFSBuddyVariable;
import com.smartfoxserver.v2.redbox.AVChatManager;
import com.smartfoxserver.v2.redbox.data.ChatSession;
import com.smartfoxserver.v2.redbox.events.RedBoxChatEvent;
import com.smartfoxserver.v2.redbox.events.RedBoxConnectionEvent;
import com.smartfoxserver.v2.requests.buddylist.AddBuddyRequest;
import com.smartfoxserver.v2.requests.buddylist.BlockBuddyRequest;
import com.smartfoxserver.v2.requests.buddylist.BuddyMessageRequest;
import com.smartfoxserver.v2.requests.buddylist.GoOnlineRequest;
import com.smartfoxserver.v2.requests.buddylist.InitBuddyListRequest;
import com.smartfoxserver.v2.requests.buddylist.RemoveBuddyRequest;
import com.smartfoxserver.v2.requests.buddylist.SetBuddyVariablesRequest;
import com.smartfoxserver.v2.util.ClientDisconnectionReason;

import components.BuddyChatPanel;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.controls.Button;
import mx.managers.PopUpManager;

[Embed(source="assets/icon_available.png")]
public static var iconAvailable:Class;

[Embed(source="assets/icon_away.png")]
public static var iconAway:Class;

[Embed(source="assets/icon_occupied.png")]
public static var iconOccupied:Class;

[Embed(source="assets/icon_offline.png")]
public static var iconOffline:Class;

[Embed(source="assets/icon_blocked.png")]
public static var iconBlocked:Class;

public static const BUDDYVAR_AGE:String = SFSBuddyVariable.OFFLINE_PREFIX + "age";
public static const BUDDYVAR_MOOD:String = "mood";

private var sfs:SmartFox;
private var alert:Alert;
private var _isBuddyListInited:Boolean;
private var buddyChatPanelsList:Array;
private var avChatMan:AVChatManager;

private function init():void
{
	// Get reference to SmartFoxServer connection
	sfs = loginPanel.connector.connection;
	
	// Add listeners
	// NOTE: for sake of simplicty, most buddy-related events cause the whole
	// buddylist in the interface to be recreated from scratch, also if those
	// events are caused by the current user himself. A more refined approach should
	// update the specific list items.
	sfs.addEventListener(SFSEvent.LOGIN, onLogin);
	sfs.addEventListener(SFSEvent.CONNECTION_LOST, onConnectionLost);
	sfs.addEventListener(SFSBuddyEvent.BUDDY_LIST_INIT, onBuddyListInit);
	sfs.addEventListener(SFSBuddyEvent.BUDDY_ERROR, onBuddyError);
	sfs.addEventListener(SFSBuddyEvent.BUDDY_ONLINE_STATE_UPDATE, onBuddyListUpdate);
	sfs.addEventListener(SFSBuddyEvent.BUDDY_VARIABLES_UPDATE, onBuddyListUpdate);
	sfs.addEventListener(SFSBuddyEvent.BUDDY_ADD, onBuddyListUpdate);
	sfs.addEventListener(SFSBuddyEvent.BUDDY_REMOVE, onBuddyListUpdate);
	sfs.addEventListener(SFSBuddyEvent.BUDDY_BLOCK, onBuddyListUpdate);
	sfs.addEventListener(SFSBuddyEvent.BUDDY_MESSAGE, onBuddyMessage);
	
	isBuddyListInited = false;
	
	buddyChatPanelsList = new Array();
	
}

[Bindable]
public function set isBuddyListInited(value:Boolean):void
{
	_isBuddyListInited = value;
}
public function get isBuddyListInited():Boolean
{
	return _isBuddyListInited;
}

//---------------------------------
// User interaction event handlers
//---------------------------------

/**
 * Set current user nickname.
 * This can be done if the current user is online in the buddy system only. 
 */
public function onSetNickBtClick():void
{
	var nick:BuddyVariable = new SFSBuddyVariable(ReservedBuddyVariables.BV_NICKNAME, details.ti_nick.text);
	sfs.send(new SetBuddyVariablesRequest([nick]));
}

/**
 * Set current user state.
 * This can be done if the current user is online in the buddy system only.
 */
public function onStateDdChange():void
{
	var state:BuddyVariable = new SFSBuddyVariable(ReservedBuddyVariables.BV_STATE, details.dd_states.selectedLabel);
	sfs.send(new SetBuddyVariablesRequest([state]));
}

/**
 * Make current user go online/offline in the buddy list system.
 */
public function onOnlineCbChange():void
{
	sfs.send(new GoOnlineRequest(details.cb_online.selected));
}

/**
 * Set current user age.
 * This can be done if the current user is online in the buddy system only.
 */
public function onSetAgeBtClick():void
{
	var age:BuddyVariable = new SFSBuddyVariable(BUDDYVAR_AGE, details.ns_age.value);
	sfs.send(new SetBuddyVariablesRequest([age]));
}

/**
 * Set current user mood.
 * This can be done if the current user is online in the buddy system only.
 */
public function onSetMoodBtClick():void
{
	var mood:BuddyVariable = new SFSBuddyVariable(BUDDYVAR_MOOD, details.ti_mood.text);
	sfs.send(new SetBuddyVariablesRequest([mood]));
}

/**
 * Add a buddy.
 */
public function onAddBuddyBtClick():void
{
	if (contactList.ti_buddyName.text != "")
	{
		sfs.send(new AddBuddyRequest(contactList.ti_buddyName.text));
		contactList.ti_buddyName.text = "";
	}
}

/**
 * Remove a buddy.
 */
public function onRemoveBuddyBtClick():void
{
	sfs.send(new RemoveBuddyRequest(contactList.ls_buddies.selectedItem.name));
}

/**
 * Block/unblock a buddy.
 */
public function onBlockBuddyBtClick():void
{
	var isBlocked:Boolean = contactList.ls_buddies.selectedItem.isBlocked;
	sfs.send(new BlockBuddyRequest(contactList.ls_buddies.selectedItem.name, !isBlocked));
}

private function onAlertClosed(evt:Event):void
{
	removeAlert();
}

private function onPopUpClosed(evt:Event):void
{
	removeAllBuddyChats();
}

/**
 * Start a chat with the buddy.
 */
public function onBuddyListDoubleClick():void
{
	var buddy:Buddy = contactList.ls_buddies.selectedItem as SFSBuddy;
	
	if (!buddy.isBlocked && buddy.isOnline)
		getBuddyChatPanel(buddy);
}

/**
 * Close buddy chat panel. 
 */
public function onBuddyChatCloseBtClick(evt:Event):void
{
	var panel:BuddyChatPanel = evt.target as BuddyChatPanel;
	removeBuddyChat(panel);
}

/**
 * Handle click on the "Send" button in the chat panel.
 */
public function onBuddyMsgSendBtClick(evt:Event):void
{
	var panel:BuddyChatPanel = (evt.target as Button).parentDocument as BuddyChatPanel
	
	if (panel.ti_message.text != "")
	{
		var params:ISFSObject = new SFSObject();
		params.putUtfString("recipient", panel.buddy.name);
		
		var request:BuddyMessageRequest = new BuddyMessageRequest(panel.ti_message.text, panel.buddy, params);
		sfs.send(request); 
		
		panel.ti_message.text = "";
	}
}

/**
 * Handle "start A/V chat" button click in the buddy chat panel.
 */
public function onStartAVChatBtClick(panel:BuddyChatPanel, t:String):void
{
	if (avChatMan != null && avChatMan.isConnected)
	{
		var buddyId:int = panel.buddy.id;
		var type:String;
		
		switch (t)
		{
			case "snd":
				type = AVChatManager.REQ_TYPE_SEND;
				break
			
			case "rcv":
				type = AVChatManager.REQ_TYPE_RECEIVE;
				break
		}
		
		var session:ChatSession = avChatMan.sendChatRequest(type, buddyId, true, true);
		
		if (session != null) // session is null if the same request to the same recipient has already been submitted before (and is still valid)
		{
			var txt:String;
			
			// Set proper session id in the chat panel, to handle the recipient's response properly:
			// "outgoingSessionId" is set when a SEND request is made (I would like to send my video stream to my buddy)
			// "incomingSessionId" is set when a RECEIVE request is made (I would like to receive the video stream of my buddy)
			if (type == AVChatManager.REQ_TYPE_SEND)
			{
				panel.outgoingSessionId = session.id;
				txt = "You have invited your buddy to watch your webcam stream";
				panel.showMyVideo(Camera.getCamera());
				enableStartMyCamButton(panel, false);
			}
			else if (type == AVChatManager.REQ_TYPE_RECEIVE)
			{
				panel.incomingSessionId = session.id;
				txt = "You have invited your buddy to start his webcam";
				enableStartBuddyCamButton(panel, false);
			}
			
			showSystemMessage(panel, txt);
		}
	}
	else
		showSystemMessage(panel, "Unable to send an A/V streaming request: the connection to RED5 server could not be established on application startup");
}

/**
 * Handle "stop A/V chat" button click in the buddy chat panel.
 */
public function onStopAVChatBtClick(panel:BuddyChatPanel, id:String):void
{
	// Send stop request
	avChatMan.stopChat(id);
	
	var txt:String;
	
	// Re-enable proper a/v buttons, remove video if necessary and display system message in chat box
	if (id == panel.incomingSessionId)
	{
		panel.stopBuddyVideo();
		enableStartBuddyCamButton(panel, true);
		txt = "You have stopped your buddy's webcam stream";
	}
	else if (id == panel.outgoingSessionId)
	{
		panel.stopMyVideo();
		enableStartMyCamButton(panel, true);
		txt = "You have stopped your webcam stream";
	}
	
	showSystemMessage(panel, txt);
}

/**
 * Handle "accept A/V chat" button click in the buddy chat panel.
 */
public function onAcceptAVRequestBtClick(panel:BuddyChatPanel, id:String):void
{
	if (avChatMan != null && avChatMan.isConnected)
	{
		try
		{
			// Send acceptance
			avChatMan.acceptChatRequest(id);
			
			var txt:String;
			
			// Disable proper a/v buttons and display system message in chat box
			if (id == panel.incomingSessionId)
			{
				panel.buddyAVButtonsCont.enabled = true;
				enableStartBuddyCamButton(panel, false);
				txt = "You have accepted to receive your buddy's webcam stream";
			}
			else if (id == panel.outgoingSessionId)
			{
				panel.myAVButtonsCont.enabled = true;
				enableStartMyCamButton(panel, false);
				txt = "You have accepted to send your webcam stream to your buddy";
			}
			
			showSystemMessage(panel, txt);
			
			// Reset the chat panel current status (to hide proper invitation)
			resetPanelStatus(panel, id);
		}
		catch (e:Error)
		{
			showAlert("The following error occurred while accepting the A/V streaming request: " + e.message);
		}
	}
	else
		showSystemMessage(panel, "Unable to accept an A/V streaming request: the connection to RED5 server could not be established on application startup");
}

/**
 * Handle "decline A/V chat" button click in the buddy chat panel.
 */
public function onDeclineAVRequestBtClick(panel:BuddyChatPanel, id:String):void
{
	try
	{
		// Send refusal
		avChatMan.refuseChatRequest(id);
		
		var txt:String;
		
		// Re-enable proper a/v buttons and display system message in chat box
		if (id == panel.incomingSessionId)
		{
			panel.buddyAVButtonsCont.enabled = true;
			txt = "You have refused to receive your buddy's webcam stream";
		}
		else if (id == panel.outgoingSessionId)
		{
			panel.myAVButtonsCont.enabled = true;
			txt = "You have refused to send your webcam stream to your buddy";
		}
		
		showSystemMessage(panel, txt)
		
		// Reset the chat panel current status (to hide proper invitation)
		resetPanelStatus(panel, id);
	}
	catch (e:Error)
	{
		showAlert("The following error occurred while declining the A/V streaming request: " + e.message);
	}
}

//---------------------------------
// SmartFoxServer event handlers
//---------------------------------

/**
 * On login, show the lobby view.
 */
private function onLogin(evt:SFSEvent):void
{
	// Move to main view, and display user name 
	mainView.selectedChild = view_main;
	details.lb_myUserName.text = sfs.mySelf.name;
	
	// Initialize buddy list system
	sfs.send(new InitBuddyListRequest());
	
	// Initialize AVChatManager
	initializeAV();
}

/**
 * On connection lost, go back to login panel view and display disconnection error message.
 */
private function onConnectionLost(evt:SFSEvent):void
{
	// Reset interface
	reset();
	
	// Remove popup, if any
	removeAllBuddyChats();
	
	// Remove alert, if displayed
	removeAlert();
	
	// Remove listeners added to AVChatManager instance
	if (avChatMan != null)
	{
		avChatMan.removeEventListener(RedBoxConnectionEvent.AV_CONNECTION_INITIALIZED, onAVConnectionInited);
		avChatMan.removeEventListener(RedBoxConnectionEvent.AV_CONNECTION_ERROR, onAVConnectionError);
		avChatMan.removeEventListener(RedBoxChatEvent.RECIPIENT_MISSING, onRecipientMissing);
		avChatMan.removeEventListener(RedBoxChatEvent.CHAT_REQUEST, onChatRequest);
		avChatMan.removeEventListener(RedBoxChatEvent.CHAT_REFUSED, onChatRefused);
		avChatMan.removeEventListener(RedBoxChatEvent.CHAT_STARTED, onChatStarted);
		avChatMan.removeEventListener(RedBoxChatEvent.CHAT_STOPPED, onChatStopped);
		avChatMan = null;
	}
	
	// Show disconnection message, unless user chose voluntarily to close the connection
	if (evt.params.reason != ClientDisconnectionReason.MANUAL)
	{
		var msg:String = "Connection lost";
		
		switch (evt.params.reason)
		{
			case ClientDisconnectionReason.IDLE:
				msg += "\nYou have exceeded the maximum user idle time";
				break;
			
			case ClientDisconnectionReason.KICK:
				msg += "\nYou have been kicked";
				break;
			
			case ClientDisconnectionReason.BAN:
				msg += "\nYou have been banned";
				break;
			
			case ClientDisconnectionReason.UNKNOWN:
				msg += " due to unknown reason\nPlease check the server-side log";
				break;
		}
		
		loginPanel.ta_error.text = msg;
	}
	
	// Show login view
	mainView.selectedChild = view_connecting;
}

/**
 * Initialize interface when buddy list data is received.
 */
private function onBuddyListInit(evt:SFSBuddyEvent):void
{
	// Populate list of buddies
	onBuddyListUpdate(evt);
	
	// Set current user details as buddy
	
	// Nick
	details.ti_nick.text = sfs.buddyManager.myNickName;
	
	// States
	var states:Array = sfs.buddyManager.buddyStates;
	details.dd_states.dataProvider = states;
	var state:String = (sfs.buddyManager.myState != null ? sfs.buddyManager.myState : "");
	if (states.indexOf(state) > -1)
		details.dd_states.selectedIndex = states.indexOf(state);
	else
		details.dd_states.selectedIndex = 0;
	
	// Online
	details.cb_online.selected = sfs.buddyManager.myOnlineState;
	
	// Buddy variables
	var age:BuddyVariable = sfs.buddyManager.getMyVariable(BUDDYVAR_AGE);
	details.ns_age.value = ((age != null && !age.isNull()) ? age.getIntValue() : 30);
	
	var mood:BuddyVariable = sfs.buddyManager.getMyVariable(BUDDYVAR_MOOD);
	details.ti_mood.text = ((mood != null && !mood.isNull()) ? mood.getStringValue() : "");
	
	isBuddyListInited = true;
}

/**
 * Build buddies list.
 */
private function onBuddyListUpdate(evt:SFSBuddyEvent):void
{
	var buddies:ArrayCollection = new ArrayCollection();
	
	for each (var buddy:Buddy in sfs.buddyManager.buddyList)
	{
		buddies.addItem(buddy);
		
		// Check if a chat panel is opened and, in case, update it
		var panel:BuddyChatPanel = buddyChatPanelsList[buddy.name];
		
		if (panel != null)
		{
			// Show a system message
			if (!evt.params.isItMe)
				showSystemMessage(panel, (buddy.isOnline ? "Your buddy is online again" : "Your buddy went offline"));
			
			var enable:Boolean = (evt.params.isItMe ? (sfs.buddyManager.myOnlineState && buddy.isOnline) : buddy.isOnline);
			panel.bt_sendMessage.enabled = enable;
			panel.bt_startBuddyCam.enabled = enable;
			panel.bt_startMyCam.enabled = enable;
			
			// If a buddy was blocked, close its chat panel
			if (buddy.isBlocked)
				removeBuddyChat(panel);
		}
	}
	
	contactList.ls_buddies.dataProvider = buddies;
}

/**
 * Message received from a buddy.
 */
private function onBuddyMessage(evt:SFSBuddyEvent):void
{
	var isItMe:Boolean = evt.params.isItMe;
	var sender:Buddy = evt.params.buddy;
	var message:String = evt.params.message;
	
	var buddy:Buddy;
	var senderName:String;
	
	if (isItMe)
	{
		var buddyName:String = (evt.params.data as ISFSObject).getUtfString("recipient");
		buddy = sfs.buddyManager.getBuddyByName(buddyName);
		senderName = sfs.mySelf.name;
	}
	else
	{
		buddy = sender;
		senderName = sender.name;
	}
	
	if (buddy != null)
	{
		var panel:BuddyChatPanel = getBuddyChatPanel(buddy);
		showBuddyMessage(panel, senderName, message);
	}
}

private function onBuddyError(evt:SFSBuddyEvent):void
{
	showAlert("The following error occurred in the buddy list system: " + evt.params.errorMessage);
}

//---------------------------------------------------------------------
// RedBox AVChatManager event handlers
//---------------------------------------------------------------------

/**
 * Handle A/V connection initialized.
 */
public function onAVConnectionInited(evt:RedBoxConnectionEvent):void
{
	// Nothing to do. Usually we should wait this event before enabling the a/v chat related interface elements.
}

/**
 * Handle A/V connection error.
 */
public function onAVConnectionError(evt:RedBoxConnectionEvent):void
{
	var error:String = "The following error occurred while trying to establish an A/V connection: " + evt.params.errorCode;
	
	// Show alert
	showAlert(error);
	
	// Re-enable proper a/v buttons, remove my video if necessary and display system message in chat box
	for each (var panel:BuddyChatPanel in buddyChatPanelsList)
	{
		if (panel.incomingSessionId != null)
		{
			panel.stopBuddyVideo();
			enableStartBuddyCamButton(panel, true);
			panel.incomingSessionId = null;
		}
		
		if (panel.outgoingSessionId != null)
		{
			panel.stopMyVideo();
			enableStartMyCamButton(panel, true);
			panel.outgoingSessionId = null;
		}
		
		showSystemMessage(panel, error);
	}
}

/**
 * Handle missing recipient error.
 */
public function onRecipientMissing(evt:RedBoxChatEvent):void
{
	// Display an error message and reset the interface (this is probably not necessary, as the interface should be disabled
	// as soon as the buddy goes offline, so sending a request to a missing recipient should not be possible).
	
	var session:ChatSession = evt.params.chatSession;
	var buddy:Buddy = sfs.buddyManager.getBuddyById(session.mateId);
	var panel:BuddyChatPanel = buddyChatPanelsList[buddy.name];
	
	if (panel != null)
	{
		if (session.requestType == AVChatManager.REQ_TYPE_SEND)
		{
			panel.outgoingSessionId = null;
			panel.stopMyVideo();
			enableStartMyCamButton(panel, true);
		}
		else if (session.requestType == AVChatManager.REQ_TYPE_RECEIVE)
		{
			panel.incomingSessionId = null;
			enableStartBuddyCamButton(panel, true);
		}
		
		showSystemMessage(panel, "Your buddy is not available");
	}
}

/**
 * Handle incoming A/V chat request.
 */
public function onChatRequest(evt:RedBoxChatEvent):void
{
	var session:ChatSession = evt.params.chatSession;
	var buddy:Buddy = sfs.buddyManager.getBuddyById(session.mateId);
	var panel:BuddyChatPanel = getBuddyChatPanel(buddy);
	
	var txt:String;
	
	// Depending on the request type, set the incoming or outgoing session id in the chat panel,
	// disable the proper a/v buttons container, display the proper system message in the chat box
	// and set the chat panel status to show the proper invitation message and accept/decline buttons
	switch (session.requestType)
	{
		case AVChatManager.REQ_TYPE_RECEIVE:
			panel.outgoingSessionId = session.id;
			panel.myAVButtonsCont.enabled = false;
			txt = "Your buddy invited you to start your webcam";
			if (panel.currentState == "invitation_receive")
				panel.currentState = "invitation_both";
			else
				panel.currentState = "invitation_send";
			break
		
		case AVChatManager.REQ_TYPE_SEND:
			panel.incomingSessionId = session.id;
			panel.buddyAVButtonsCont.enabled = false;
			txt = "Your buddy invited you to watch his/her webcam stream";
			if (panel.currentState == "invitation_send")
				panel.currentState = "invitation_both";
			else
				panel.currentState = "invitation_receive";
			break
	}
	
	showSystemMessage(panel, txt);
}

/**
 * Handle A/V chat started event.
 */
public function onChatStarted(evt:RedBoxChatEvent):void
{
	var session:ChatSession = evt.params.chatSession;
	var buddy:Buddy = sfs.buddyManager.getBuddyById(session.mateId);
	var panel:BuddyChatPanel = getBuddyChatPanel(buddy);
	
	// If my stream is not null, I'm transmitting, so I have to show my own video
	if (session.myStream != null && session.enableCamera)
		panel.showMyVideo(Camera.getCamera());
	
	// Attach my mate' stream to the video object
	if (session.mateStream != null && session.enableCamera)
		panel.showBuddyVideo(session.mateStream);
	
	if (session.iAmRequester)
	{
		var txt:String;
		
		// Write a system message in the chat box
		if (session.id == panel.outgoingSessionId)
			txt = "Your buddy accepted to receive your webcam stream";
		else if (session.id == panel.incomingSessionId)
			txt = "Your buddy accepted to transmit his a/v stream";
		
		showSystemMessage(panel, txt);
	}
}

/**
 * Handle A/V chat refused event.
 */
public function onChatRefused(evt:RedBoxChatEvent):void
{
	var session:ChatSession = evt.params.chatSession;
	var buddy:Buddy = sfs.buddyManager.getBuddyById(session.mateId);
	var panel:BuddyChatPanel = getBuddyChatPanel(buddy);
	
	var txt:String;
	
	// Write a system message in the chat box, re-enable buttons and stop my own video
	if (session.id == panel.outgoingSessionId)
	{
		panel.stopMyVideo();
		enableStartMyCamButton(panel, true);
		txt = "Your buddy refused to receive your webcam stream";
	}
	else if (session.id == panel.incomingSessionId)
	{
		enableStartBuddyCamButton(panel, true);
		txt = "Your buddy refused to transmit his/her webcam stream";
	}
	
	showSystemMessage(panel, txt);
}

/**
 * Handle A/V chat stopped event.
 */
public function onChatStopped(evt:RedBoxChatEvent):void
{
	var session:ChatSession = evt.params.chatSession;
	var buddy:Buddy = sfs.buddyManager.getBuddyById(session.mateId);
	var panel:BuddyChatPanel = getBuddyChatPanel(buddy);
	
	var txt:String;
	
	// Re-enable proper a/v buttons, remove my video if necessary and display system message in chat box
	if (session.id == panel.incomingSessionId)
	{
		if (panel.currentState == "invitation_both")
			panel.currentState = "invitation_send";
		else if (panel.currentState == "invitation_receive")
			panel.currentState = "";
		else
			panel.stopBuddyVideo();
		
		panel.buddyAVButtonsCont.enabled = true;
		enableStartBuddyCamButton(panel, true);
		txt = "Your buddy stopped his webcam stream";
	}
	else if (session.id == panel.outgoingSessionId)
	{
		if (panel.currentState == "invitation_both")
			panel.currentState = "invitation_receive";
		else if (panel.currentState == "invitation_send")
			panel.currentState = "";
		else
			panel.stopMyVideo();
		
		panel.myAVButtonsCont.enabled = true;
		enableStartMyCamButton(panel, true);
		txt = "Your buddy stopped your webcam stream";
	}
	
	showSystemMessage(panel, txt);
}

//---------------------------------
// Other methods
//---------------------------------

private function reset():void
{
	isBuddyListInited = false;
	contactList.ls_buddies.dataProvider = null;
}

/**
 * Initialize the AVChatManager instance.
 */
private function initializeAV():void
{
	// Create AVChatmanager instance
	avChatMan = new AVChatManager(sfs, sfs.currentIp, false, true);
	
	avChatMan.addEventListener(RedBoxConnectionEvent.AV_CONNECTION_INITIALIZED, onAVConnectionInited);
	avChatMan.addEventListener(RedBoxConnectionEvent.AV_CONNECTION_ERROR, onAVConnectionError);
	avChatMan.addEventListener(RedBoxChatEvent.RECIPIENT_MISSING, onRecipientMissing);
	avChatMan.addEventListener(RedBoxChatEvent.CHAT_REQUEST, onChatRequest);
	avChatMan.addEventListener(RedBoxChatEvent.CHAT_REFUSED, onChatRefused);
	avChatMan.addEventListener(RedBoxChatEvent.CHAT_STARTED, onChatStarted);
	avChatMan.addEventListener(RedBoxChatEvent.CHAT_STOPPED, onChatStopped);
}

/**
 * Retrieve the buddy chat panel.
 * If not existing, create it.
 */
private function getBuddyChatPanel(buddy:Buddy):BuddyChatPanel
{
	// Check if private chat with existing user is already in progress
	var panel:BuddyChatPanel = buddyChatPanelsList[buddy.name];
	
	if (panel == null)
	{
		panel = new BuddyChatPanel();
		panel.buddy = buddy;
		panel.title = "Chat with " + buddy.name;
		
		// Add panel to display list
		view_main.addChild(panel);
		
		// Set panel position
		randomMovePanel(panel);
		
		// Keep a reference to the panel for later use
		buddyChatPanelsList[buddy.name] = panel;
	}
	
	return panel;
}

/**
 * Slightly alter the position of the panel once centered.
 */
private function randomMovePanel(panel:BuddyChatPanel):void
{
	panel.x = Math.round((this.width - 560) / 2);
	panel.y = Math.round((this.height - 366) / 2);
	
	var dx:int = int(Math.random() * 20);
	var dy:int = int(Math.random() * 20);
	var sx:int = Math.random() * 100 > 49 ? 1 : -1;
	var sy:int = Math.random() * 100 > 49 ? -1 : 1;
	
	panel.x += dx * sx;
	panel.y += dy * sy;
}

/**
 * Remove the buddy chat panel.
 */
private function removeBuddyChat(panel:BuddyChatPanel):void
{
	// Stop A/V connections
	if (panel.incomingSessionId != null)
		avChatMan.stopChat(panel.incomingSessionId);
	if (panel.outgoingSessionId != null)
		avChatMan.stopChat(panel.outgoingSessionId);
	
	view_main.removeChild(panel);
	
	// Remove from list
	delete buddyChatPanelsList[panel.buddy.name];
}

/**
 * Show buddy message in chat panel.
 */
private function showBuddyMessage(panel:BuddyChatPanel, senderName:String, txt:String):void
{
	panel.ta_chat.htmlText += "<b><font color='#F88121'>" + senderName + ": </font></b>" + txt;
	panel.ta_chat.verticalScrollPosition = panel.ta_chat.maxVerticalScrollPosition;
}

/**
 * Show system message in buddy chat panel.
 */
private function showSystemMessage(panel:BuddyChatPanel, txt:String):void
{
	panel.ta_chat.htmlText += "<i><font color='#666666'>" + txt + "</font></i>\n";
	panel.ta_chat.verticalScrollPosition = panel.ta_chat.maxVerticalScrollPosition;
}

private function enableStartMyCamButton(panel:BuddyChatPanel, enable:Boolean):void
{
	var online:Boolean = sfs.buddyManager.myOnlineState && panel.buddy.isOnline;
	
	panel.bt_startMyCam.enabled = (online && enable);
	panel.bt_stopMyCam.enabled = (online && !enable);
}

private function enableStartBuddyCamButton(panel:BuddyChatPanel, enable:Boolean):void
{
	var online:Boolean = sfs.buddyManager.myOnlineState && panel.buddy.isOnline;
	
	panel.bt_startBuddyCam.enabled = (online && enable);
	panel.bt_stopBuddyCam.enabled = (online && !enable);
}

private function resetPanelStatus(panel:BuddyChatPanel, id:String):void
{
	if (panel.currentState != "invitation_both")
		panel.currentState = "";
	else
	{
		if (id == panel.incomingSessionId)
			panel.currentState = "invitation_send";
		if (id == panel.outgoingSessionId)
			panel.currentState = "invitation_receive";
	}
}

private function showAlert(message:String):void
{
	// Remove previous alert, if any
	removeAlert()
	
	// Show alert
	alert = Alert.show(message, "Warning", Alert.OK, null, onAlertClosed);
}

private function removeAlert():void
{
	if (alert != null)
		PopUpManager.removePopUp(alert);
	
	alert = null;
}

private function removeAllBuddyChats():void
{
	// Remove buddy chat panels
	for each (var panel:BuddyChatPanel in buddyChatPanelsList)
		removeBuddyChat(panel);
}